﻿using System;
using ThinkGeo.MapSuite.Core;

namespace PointSnapToLayer
{
	public struct CenterTsquare
	{
		public PointShape CenterPointShape;
		public bool IsFixed;
	}

	public class Tsquare
	{
		/// <summary>
		/// Центральная точка линий
		/// </summary>
		public CenterTsquare CenterTsquare;

		#region data

	    public LineShape HorizontalSlideway;
	    public LineShape VerticalSlideway;

		#endregion

        #region Additional

        #region Angles world

        /// <summary>
        /// Получить угол, обратный данному
        /// </summary>
        /// <param name="angle">Исходный угол</param>
        /// <returns>Возвращаем угол, обратный данному</returns>
        private double GetReverseAngle(double angle)
        {
            return angle + 180 > 360 ? angle - 180 : angle + 180;
        }

        /// <summary>
        /// Получить угол между двумя вершинами (через тангенс и проекцию на ось у)
        /// </summary>
        /// <param name="fromVertex">Вершина ОТ</param>
        /// <param name="toVertex">Вершина К КОТОРОЙ</param>
        /// <returns></returns>
        private double GetAngleFromTwoVertices(Vertex fromVertex, Vertex toVertex)
        {
            double alpha;

            if (Math.Abs(fromVertex.X - toVertex.X) > 0.000000000000001)
            {
                double tangentAlpha = (fromVertex.Y - toVertex.Y) / (toVertex.X - fromVertex.X);
                alpha = Math.Atan(tangentAlpha) * 180 / Math.PI;
                if (alpha < 0)
                {
                    alpha += 360;
                }
            }
            else
            {
                alpha = (fromVertex.Y > toVertex.Y) ? 90 : 270;
            }

            return alpha;
        }

        #endregion

        /// <summary>
        /// Обновление линий
        /// </summary>
        public void RefreshSlideways()
        {
            #region Create perpendicular line

            var segmentMultiLineShape = new LineShape(new[]
			                                          	{
			                                          		new Vertex(CenterTsquare.CenterPointShape.X, CenterTsquare.CenterPointShape.Y),
			                                          		new Vertex(VerticalSlideway.Vertices[1].X, VerticalSlideway.Vertices[1].Y)
			                                          	});

            var pointShape1 = new PointShape(segmentMultiLineShape.Vertices[0]);
            double angle1 = GetAngleFromTwoVertices(segmentMultiLineShape.Vertices[0], segmentMultiLineShape.Vertices[1]);
            double offsetAngle1A = angle1;
            double offsetAngle1B = GetReverseAngle(angle1);

            var offsetPointShape1A = (PointShape)pointShape1.CloneDeep();
            var offsetPointShape1B = (PointShape)pointShape1.CloneDeep();

            offsetPointShape1A.TranslateByDegree(1, offsetAngle1A);
            offsetPointShape1B.TranslateByDegree(1, offsetAngle1B);

            HorizontalSlideway = new LineShape(
                new[]
					{
						new Vertex(offsetPointShape1A.X, offsetPointShape1A.Y),
						new Vertex(offsetPointShape1B.X, offsetPointShape1B.Y)
					}
                );
            #endregion
        }

        #endregion
	}
}